/* Copyright (C) 2003 Timo Sirainen */

#include "lib.h"
#include "istream-lob.h"
#include "oracle-storage.h"
#include "oracle-mail.h"

static const struct mail_full_flags *get_flags(struct mail *_mail)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	data->flags.flags = 0;

	if (mail->answered == 'Y')
		data->flags.flags |= MAIL_ANSWERED;
	if (mail->flagged == 'Y')
		data->flags.flags |= MAIL_FLAGGED;
	if (mail->seen == 'Y')
		data->flags.flags |= MAIL_SEEN;
	if (mail->deleted == 'Y')
		data->flags.flags |= MAIL_DELETED;
	if (mail->draft == 'Y')
		data->flags.flags |= MAIL_DRAFT;

	return &data->flags;
}

static const struct message_part *get_parts(struct mail *_mail)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return NULL;
}

static time_t get_received_date(struct mail *_mail)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return (time_t)-1;
}

static time_t get_date(struct mail *_mail, int *timezone)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return (time_t)-1;
}

static uoff_t get_size(struct mail *_mail)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return (uoff_t)-1;
}

static const char *get_header(struct mail *_mail, const char *field)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return NULL;
}

static const struct message_address *
get_address(struct mail *_mail, const char *field)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;

	return NULL;
}

static const char *get_first_mailbox(struct mail *_mail, const char *field)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return NULL;
}

static struct istream *get_stream(struct mail *_mail,
				  struct message_size *hdr_size,
				  struct message_size *body_size)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return i_stream_create_lob(mail->obox->ostorage->dbh,
				   mail->lob, system_pool);
}

static const char *get_special(struct mail *_mail, enum mail_fetch_field field)
{
	struct oracle_mail *mail = (struct oracle_mail *) _mail;
	struct oracle_mail_data *data = &mail->data;

	return NULL;
}

static struct mail oracle_mail = {
	0, 0, 0,

	get_flags,
	get_parts,
	get_received_date,
	get_date,
	get_size,
	get_header,
	get_address,
	get_first_mailbox,
	get_stream,
	get_special
};

void oracle_mail_init(struct oracle_mailbox *obox, struct oracle_mail *mail)
{
	mail->mail = oracle_mail;
	mail->obox = obox;
}

int oracle_mail_next(struct oracle_mail *mail)
{
	mail->mail.seq = oracle_mailbox_uid2seq(mail->obox, mail->mail.uid);
	return 1;
}

void oracle_mail_deinit(struct oracle_mail *mail)
{
}
