/* Copyright (C) 2003 Timo Sirainen */

#include "lib.h"
#include "buffer.h"
#include "oracle-storage.h"

unsigned int oracle_mailbox_uid2seq(struct oracle_mailbox *obox,
				    unsigned int uid)
{
	const unsigned char *uid_arr;
	unsigned int lo, hi, mid, val;

	uid_arr = buffer_get_data(obox->uids, NULL);

	lo = 0;
	hi = obox->synced_messages - 1;

	while (lo < hi - 1) {
		mid = (lo + hi) / 2;
		val = uid_arr[mid];
		if (val < uid)
			lo = mid;
		else if (val > uid)
			hi = mid;
		else
			return mid + 1;
	}

	i_panic("UID %u not found", uid);
	return 0;
}

int oracle_storage_close(struct mailbox *box)
{
	struct oracle_mailbox *obox = (struct oracle_mailbox *)box;

	buffer_free(obox->uids);
	return TRUE;
}

int oracle_storage_get_status(struct mailbox *box,
			      enum mailbox_status_items items,
			      struct mailbox_status *status)
{
	struct oracle_mailbox *obox = (struct oracle_mailbox *)box;
	sqlo_stmt_handle_t sth;
	unsigned int last_nonrecent_uid;

	memset(status, 0, sizeof(*status));
	sth = orasql_prepare(obox->ostorage,
			     "SELECT messages_count, "
			     "messages_count - seen_messages_count, "
			     "last_nonrecent_uid, uid_validity, next_uid "
			     "FROM mailboxes WHERE id = %u:id", obox->id);
	sth = orasql_define_output(obox->ostorage, sth, "%u%u%u%u%u",
				   &status->messages, &status->unseen,
				   &last_nonrecent_uid,
				   &status->uidvalidity, &status->uidnext);
	if (!orasql_mailbox_fetch_one(obox, sth))
		return FALSE;

	if ((items & STATUS_RECENT) != 0) {
		sth = orasql_prepare(obox->ostorage,
				     "SELECT COUNT(*) FROM mails "
				     "WHERE mailbox_id = %u:id AND "
				     "imap_uid > %u:imapuid",
				     obox->id, last_nonrecent_uid);
		sth = orasql_define_output(obox->ostorage, sth, "%u",
					   &status->recent);
		if (!orasql_mailbox_fetch_one(obox, sth))
			return FALSE;
	}

	if ((items & STATUS_FIRST_UNSEEN_SEQ) != 0) {
		sth = orasql_prepare(obox->ostorage,
			"SELECT COUNT(*) FROM mails m WHERE mailbox_id = %u:id "
			"AND imap_uid <= (SELECT MIN(imap_uid) FROM mails "
			" WHERE mailbox_id = m.mailbox_id AND seen = 'N')",
			obox->id);
		sth = orasql_define_output(obox->ostorage, sth, "%u",
					   &status->first_unseen_seq);
		if (!orasql_mailbox_fetch_one(obox, sth))
			return FALSE;
	}

	if ((items & STATUS_CUSTOM_FLAGS) != 0) {
		/* FIXME: get them */
		status->custom_flags_count = 0;
	}

	return TRUE;
}

int oracle_storage_expunge(struct mailbox *box, int notify)
{
	struct oracle_mailbox *obox = (struct oracle_mailbox *)box;
	int rows;

	rows = orasql_exec(obox->ostorage,
			   "DELETE FROM mails WHERE mailbox_id = %u:id AND "
			   "deleted = 'Y'", obox->id);
	if (rows < 0)
		return FALSE;

	if (orasql_exec(obox->ostorage, "UPDATE mailboxes "
			"SET next_expungeseq = next_expungeseq + 1, "
			"messages_count = messages_count - %u:count "
			"WHERE id = %u:id", rows, obox->id) < 0)
		return FALSE;

	if (orasql_commit(obox->ostorage) < 0)
		return FALSE;

	return notify ? oracle_storage_sync(box, TRUE) : TRUE;
}

int oracle_storage_update_flags(struct mailbox *box, const char *messageset,
				int uidset, const struct mail_full_flags *flags,
				enum modify_type modify_type, int notify,
				int *all_found)
{
	return FALSE;
}

int oracle_storage_copy(struct mailbox *box, struct mailbox *destbox,
			const char *messageset, int uidset)
{
	return FALSE;
}
