#ifndef __ORACLE_STORAGE_H
#define __ORACLE_STORAGE_H

#include "mail-storage.h"
#include "sqlora.h"

struct oracle_storage {
	struct mail_storage storage;

	sqlo_db_handle_t dbh;
	unsigned int user_id;
};

struct oracle_mailbox {
	struct mailbox box;
	struct mail_storage *storage;
	struct oracle_storage *ostorage;

	buffer_t *uids;
	unsigned int synced_messages;
	unsigned int next_expungeseq, next_uid, next_modseq;

	unsigned int id;
};

extern struct mail_storage oracle_storage;
extern struct mailbox oracle_mailbox;

#include "oracle-sql.h"

unsigned int oracle_mailbox_uid2seq(struct oracle_mailbox *obox,
				    unsigned int uid);
int oracle_messageset_build(struct oracle_mailbox *obox, string_t *str,
			    const char *messageset, int uidset,
			    int skip_expunged);

struct mailbox_list_context *
oracle_list_mailbox_init(struct mail_storage *storage, const char *mask,
			 enum mailbox_list_flags flags, int *sorted);
int oracle_list_mailbox_deinit(struct mailbox_list_context *ctx);
struct mailbox_list *oracle_list_mailbox_next(struct mailbox_list_context *ctx);

int oracle_set_subscribed(struct mail_storage *storage,
			  const char *name, int set);

int oracle_storage_close(struct mailbox *box);
int oracle_storage_get_status(struct mailbox *box,
			      enum mailbox_status_items items,
			      struct mailbox_status *status);
int oracle_storage_sync(struct mailbox *box, int sync_expunges);
void oracle_storage_auto_sync(struct mailbox *box,
			      enum mailbox_sync_type sync_type,
			      unsigned int min_newmail_notify_interval);
int oracle_storage_expunge(struct mailbox *box, int notify);
int oracle_storage_update_flags(struct mailbox *box, const char *messageset,
				int uidset, const struct mail_full_flags *flags,
				enum modify_type modify_type, int notify,
				int *all_found);
int oracle_storage_copy(struct mailbox *box, struct mailbox *destbox,
			const char *messageset, int uidset);

struct mail_fetch_context *
oracle_storage_fetch_init(struct mailbox *box,
			  enum mail_fetch_field wanted_fields, int *update_seen,
			  const char *messageset, int uidset);
int oracle_storage_fetch_deinit(struct mail_fetch_context *ctx, int *all_found);
struct mail *oracle_storage_fetch_next(struct mail_fetch_context *ctx);

struct mail *oracle_storage_fetch_uid(struct mailbox *box, unsigned int uid,
				      enum mail_fetch_field wanted_fields);
struct mail *oracle_storage_fetch_seq(struct mailbox *box, unsigned int seq,
				      enum mail_fetch_field wanted_fields);

int oracle_storage_search_get_sorting(struct mailbox *box,
				      enum mail_sort_type *sort_program);
struct mail_search_context *
oracle_storage_search_init(struct mailbox *box, const char *charset,
			   struct mail_search_arg *args,
			   const enum mail_sort_type *sort_program,
			   enum mail_fetch_field wanted_fields,
			   const char *const wanted_headers[]);
int oracle_storage_search_deinit(struct mail_search_context *ctx);
struct mail *oracle_storage_search_next(struct mail_search_context *ctx);

struct mail_save_context *
oracle_storage_save_init(struct mailbox *box, int transaction);
int oracle_storage_save_deinit(struct mail_save_context *ctx, int rollback);
int oracle_storage_save_next(struct mail_save_context *ctx,
			     const struct mail_full_flags *flags,
			     time_t received_date, int timezone_offset,
			     struct istream *data);

#endif
