/* Copyright (c) 2003 Timo Sirainen */

#include "lib.h"
#include "ostream-lob.h"
#include "ostream-internal.h"

struct lob_ostream {
	struct _ostream ostream;

	sqlo_db_handle_t dbh;
	sqlo_lob_desc_t lob;
};

static void _close(struct _iostream *stream)
{
	struct lob_ostream *lstream = (struct lob_ostream *) stream;

	o_stream_flush(&lstream->ostream.ostream);
}

static void _destroy(struct _iostream *stream __attr_unused__)
{
}

static void _set_max_buffer_size(struct _iostream *stream __attr_unused__,
				 size_t max_size __attr_unused__)
{
}

static void _set_blocking(struct _iostream *stream __attr_unused__,
			  int timeout_msecs __attr_unused__,
			  void (*timeout_cb)(void *) __attr_unused__,
			  void *context __attr_unused__)
{
}

static void _cork(struct _ostream *stream __attr_unused__)
{
}

static int _flush(struct _ostream *stream)
{
	struct lob_ostream *lstream = (struct lob_ostream *) stream;
	int status;

	status = sqlo_lob_write_buffer(lstream->dbh, lstream->lob, 0,
				       NULL, 0, SQLO_LAST_PIECE);
	if (status != SQLO_SUCCESS) {
		i_error("sqlo_lob_write_buffer(SQLO_LAST_PIECE) "
			"failed: %s",  sqlo_geterror(lstream->dbh));
		stream->ostream.stream_errno = EIO;
		return -1;
	}
	return 1;
}

static int _have_space(struct _ostream *stream __attr_unused__,
		       size_t size __attr_unused__)
{
	return 1;
}

static int _seek(struct _ostream *stream __attr_unused__,
		 uoff_t offset __attr_unused__)
{
	stream->ostream.stream_errno = EINVAL;
	return -1;
}

static ssize_t _send(struct _ostream *stream, const void *data, size_t size)
{
	struct lob_ostream *lstream = (struct lob_ostream *) stream;
	int piece, status;

	i_assert(size <= SSIZE_T_MAX);

	if (stream->ostream.offset == 0)
		piece = SQLO_FIRST_PIECE;
	else
		piece = SQLO_NEXT_PIECE;

	status = sqlo_lob_write_buffer(lstream->dbh, lstream->lob, 0,
				       (void *) data, size, piece);
	if (status != SQLO_SUCCESS && status != SQLO_NEED_DATA) {
		i_error("sqlo_lob_write_buffer(%"PRIuUOFF_T" + %"PRIuSIZE_T") "
			"failed: %s", stream->ostream.offset, size,
			sqlo_geterror(lstream->dbh));
		stream->ostream.stream_errno = EIO;
		return -1;
	}

	stream->ostream.stream_errno = 0;
	stream->ostream.offset += size;
	return (ssize_t)size;
}

static off_t _send_istream(struct _ostream *outstream __attr_unused__,
			   struct istream *instream __attr_unused__)
{
	outstream->ostream.stream_errno = EINVAL;
	return -1;
}

struct ostream *
o_stream_create_lob(sqlo_db_handle_t dbh, sqlo_lob_desc_t lob, pool_t pool)
{
	struct lob_ostream *lstream;
	struct ostream *ostream;

	lstream = p_new(pool, struct lob_ostream, 1);
	lstream->dbh = dbh;
	lstream->lob = lob;

	lstream->ostream.iostream.close = _close;
	lstream->ostream.iostream.destroy = _destroy;
	lstream->ostream.iostream.set_max_buffer_size = _set_max_buffer_size;
	lstream->ostream.iostream.set_blocking = _set_blocking;

	lstream->ostream.cork = _cork;
	lstream->ostream.flush = _flush;
	lstream->ostream.have_space = _have_space;
	lstream->ostream.seek = _seek;
	lstream->ostream.send = _send;
	lstream->ostream.send_istream = _send_istream;

	ostream = _o_stream_create(&lstream->ostream, pool);

	return ostream;
}
